<% if params.key?("error") %>
  <div class="alert alert-danger">
    <%= params["error"] %>
  </div>
<% end %>

<div class="card">
  <h4 class="card-header">Add Feature</h4>
  <div class="card-body">
    <form action="<%= script_name %>/features" method="post" class="form-inline mb-2">
      <%== csrf_input_tag %>
      <input type="text" name="value" size="30" placeholder="search, new_pricing, etc." autofocus class="form-control mr-2 mb-2 mb-md-0">
      <input type="submit" value="Add Feature" class="btn btn-primary">
    </form>
    <p class="text-muted">
      Recommended naming conventions: lower case, <a href="https://en.wikipedia.org/wiki/Snake_case">snake case</a>, underscores over dashes.
      <strong>Good</strong>: foo_bar, foo.
      <strong>Bad</strong>: FooBar, Foo Bar, foo bar, foo-bar.
    </p>
  </div>
</div>
